<?php

namespace Drupal\ckeditor_bs_grid\Plugin\CKEditorPlugin;

use Drupal\ckeditor\CKEditorPluginBase;
use Drupal\ckeditor\CKEditorPluginConfigurableInterface;
use Drupal\ckeditor\CKEditorPluginCssInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\editor\Entity\Editor;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the "bs_grid" plugin.
 *
 * @CKEditorPlugin(
 *   id = "bs_grid",
 *   label = @Translation("Bootstrap Grid")
 * )
 */
class BsGrid extends CKEditorPluginBase implements CKEditorPluginConfigurableInterface, CKEditorPluginCssInterface, ContainerFactoryPluginInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getButtons() {
    $path = drupal_get_path('module', 'ckeditor_bs_grid') . '/js/plugins/bs_grid';
    return [
      'bs_grid' => [
        'label' => 'Bootstrap Grid',
        'image' => $path . '/icons/bs_grid.png',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFile() {
    return drupal_get_path('module', 'ckeditor_bs_grid') . '/js/plugins/bs_grid/plugin.js';
  }

  /**
   * {@inheritdoc}
   */
  public function getConfig(Editor $editor) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state, Editor $editor) {
    $settings = $editor->getSettings();
    $config = $settings['plugins']['bs_grid'] ?? [];
    $form['use_cdn'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use BS CDN'),
      '#description' => $this->t('If your theme utilizing CKEditor does not include bootstrap grid classes, or pass them via "ckeditor_stylesheets" then you can include it here. This will ONLY include it for ckeditor.'),
      '#default_value' => $config['use_cdn'] ?? TRUE,
    ];

    $form['cdn_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CDN URL'),
      '#description' => $this->t('The URL to your Bootstrap CDN, default is for grid-only.'),
      '#default_value' => $config['cdn_url'] ?? 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/5.0.2/css/bootstrap-grid.min.css',
    ];

    $form['available_columns'] = [
      '#title' => $this->t('Allowed Columns'),
      '#type' => 'checkboxes',
      '#options' => array_combine($r = range(1, 12), $r),
      '#default_value' => $config['available_columns'] ?? [],
      '#prefix' => '<div class="container-inline">',
      '#suffix' => '</div>',
      '#required' => TRUE,
    ];

    $bs_breakpoints = $this->configFactory->get('ckeditor_bs_grid.settings')->get('breakpoints');
    $breakpoint_options = [];
    foreach ($bs_breakpoints as $class => $breakpoint) {
      $breakpoint_options[$class] = $breakpoint['label'];
    }

    $form['available_breakpoints'] = [
      '#title' => $this->t('Allowed Breakpoints'),
      '#type' => 'checkboxes',
      '#options' => $breakpoint_options,
      '#default_value' => $config['available_breakpoints'] ?? [],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries(Editor $editor) {
    return [
      'core/jquery',
      'core/drupal',
      'core/drupal.ajax',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getCssFiles(Editor $editor) {
    $settings = $editor->getSettings();
    $config = $settings['plugins']['bs_grid'] ?? [];
    return !empty($config['use_cdn']) && !empty($config['cdn_url']) ? [$config['cdn_url']] : [];
  }

}
